@@ -324,4 +324,62 @@ load_fst_or_h5 <- function(
 }
 
 
+#' @name convert-fst
+#' @title Convert 'fst' files to other formats
+#' @description 'HDF5', 'csv' are common file formats that can be easily
+#' read into 'Matlab' or 'Python'
+#' @param fst_path path to 'fst' file
+#' @param hdf5_path path to 'HDF5' file; if file exists before the conversion,
+#' the file will be erased first. Please make sure the files are backed up.
+#' @param csv_path path to 'csv' file; if file exists before the conversion,
+#' the file will be erased first. Please make sure the files are backed up.
+#' @param exclude_names table names to exclude
+#' @returns \code{convert_fst_to_hdf5} will return a list of data saved to 'HDF5';
+#' \code{convert_fst_to_csv} returns the normalized 'csv' path.
+#' @export
+convert_fst_to_hdf5 <- function(fst_path, hdf5_path, exclude_names = NULL) {
+  # DIPSAUS DEBUG START
+  # fst_path <- "~/rave_data/data_dir/demo/_project_data/power_explorer/exports/KC_demo_export-20210525-071414.fst"
+  # hdf5_path <- tempfile()
 
+  tbl <- fst::fst(fst_path)
+
+  if(file.exists(hdf5_path)) {
+    unlink(hdf5_path)
+  }
+
+  nms <- names(tbl)
+  nms <- nms[!nms %in% exclude_names]
+
+  for(nm in nms) {
+    dat <- tbl[[nm]]
+    if(is.factor(dat)) { dat <- as.character(dat) }
+    compress_level <- 9
+    if(is.numeric(dat)) {
+      compress_level <- 4
+    }
+    save_h5(x = dat, file = hdf5_path, name = nm, level = compress_level, replace = TRUE)
+  }
+
+  read_mat2(hdf5_path, ram = FALSE)
+}
+
+#' @rdname convert-fst
+#' @export
+convert_fst_to_csv <- function(fst_path, csv_path, exclude_names = NULL) {
+  # DIPSAUS DEBUG START
+  # fst_path <- "~/rave_data/data_dir/demo/_project_data/power_explorer/exports/KC_demo_export-20210525-071414.fst"
+  # csv_path <- tempfile()
+
+  tbl <- fst::fst(fst_path)
+
+  if(file.exists(csv_path)) {
+    unlink(csv_path)
+  }
+
+  nms <- names(tbl)
+  nms <- nms[!nms %in% exclude_names]
+
+  utils::write.csv(tbl[, nms], file = csv_path, row.names = FALSE)
+  invisible(normalizePath(csv_path))
+}
